#include "sut.h"
#include "fpscommon.h"

int PRINT;
int KILL;
int EXEC;
int NICE;
int SORT;
struct entry entry_list[10];
int STACK[3];
int S_INDEX;

/*D
   fps - Perform a process-space find

   Syntax:
   fps [ match-expession ] [ action ]
   
   A match-expression is made up of switches and operators.

   Switches:
	-n name

	Match all processes whose COMMAND is the same as name.

	-o owner

	Match all processes whose USER is the same as owner.

	-pty term

	Match all processes whose TTY is the same as term.

	-r state

	Match all processes whose STAT is the same as state.
        Typical states are R (Run), S (Sleeping), and I (Idle).

	-rtime mm:ss

	Match all processes that have been running mm:ss time
	or longer (minutes:seconds).

	-stime dd:hh:mm

	Match all processes that have started dd days, hh hours
	and mm minutes ago.

        UNSUPPORTED
$       -cpu percentage
$       -mem percentage
        

    Operators:
	-and

	Perform a logical AND on two switches.

	-or

re	Perform a logical OR on two switches.

	\!

	Perform a logical NOT on a single switch.

	\(, \)

	Switches can be grouped with parenthesis.  Also a \! can be
	applied to two switches grouped with parenthesis.

	Note that these have been shown quoted, as required for the
	common shells.

    Options:
	-kill signame

	Kill the process that matches all criteria.  This switch
	should not be combined with the -id or the -print switches.
	You must be the owner or superuser.  signame may be either the
        signal number or the name (e.g., SIGINT).

	-nice n

	Set the nice value of matched jobs to n.  +n increments (makes nicer)
        and -n decrements.  
	You must be the owner or superuser.

	-print

	Display the full line from the ps command of all processes
	that match the selected criteria.

	-tn

	Causes only the tailname of the COMMAND to be printed.

	NOTE-THIS WILL CHANGE TO ACT LIKE -n name (i.e., -tn name)

    Examples:
.vb
   fps -rtime 1:00 -and \! -o root -print
.ve
       Prints information on those jobs that have used more than 1 minute
       of CPU time and that are not owned by root.
.vb
   fps -stime 1:0:0 -and \! -o root -print
.ve
       Prints information on those jobs that were started more than 1
       day ago and that are not owned by root.  
.vb
   fps -o $LOGNAME -tn -n bigjob -kill SIGINT
.ve
       Kills all processes named 'bigjob' owned by the logged-in user
.vb
   fps -o $LOGNAME -tn -n bigjob -exec xterm -e dbx bigjob \{\} \;
.ve
       Runs dbx in an xterm on all jobs named 'bigjob'  (OSF users may 
       need \code{dbx -pid \{\} bigjob} instead).  

    Notes:
    Not yet fully implemented.  Requires the output from /bin/ps to be 
    parseable; may not understand some versions of ps.
D*/
main(argc, argv)
int argc;
char **argv;
{
SUT_Jobs    jobs;
SUT_Range   *ranges;
int         nranges;
char        *(*HostnumToName)();
void        *HNctx;
char        *(*ExeNameF)();
void        *ExeCtx;
char        **(*ArgsF)();
void        *ArgsC();
char        *Environment;
extern char *SUT_HostName();
int         err;
char        prun_path[1024];
int         *host_nums, count;
char        *(Argv[3]);
int length = 1, buf_index = 0, tail = 0;
char buf[LINEBUFFSIZE], **swit = NULL, pchar[1];
char **oldargv;
struct ps_line one_line;

SUT_Get_debug();

/* This will handle any SIGCHLDs (made global so that there are 
   no race conditions from one call to the next) 
 */
SUT_Init( &jobs );
SUT_Set_jobs_data( &jobs );

/* Here is where we should run the local program (argv[2] on) in a 
   subprocess, adding its stdin/stdout/stderr to the "jobs" structure 
 */
Argv[0] = "/bin/ps";
#ifdef PS_SYSV
Argv[1] = "-elf";
#else
Argv[1] = "auxww";
#endif
Argv[2] = 0;
if (err = SUT_Startprocess( &jobs, Argv )) {
    fprintf( stderr, "Error in starting ps program\n" );
    SUT_Errormessage( stderr, err );
    exit(1);
    }

/* Load in the command line parameters */
oldargv = argv + 1;
while(*oldargv != NULL)
   get_token(&oldargv);

/* Check the command line.  Also sets things like PRINT */
valid_command_line(argv, &tail);

/* Setup the evaluation stack for evaluating whether the process matches
   the given conditions */
fill_entry_list();

while(length) {
    while(((length = read(jobs.conn[0].readfd, pchar, 1)) == 1) && 
	  (*pchar != '\n'))
	buf[buf_index++] = *pchar;
    buf[buf_index] = (char)NULL;
    buf_index = 0;
    get_process_line(buf, &one_line);
    if(evaluate_process_line(&one_line)) {
	if(PRINT)
	    print_process_line(&one_line);
	else
	    /* This routine handles the other options (kill, nice, exec) */
	    print_pids(&one_line);
	}
    reset_process_line(&one_line);
    }

return 0;
}
