#include "sut.h"

/* 
   This file contains a program that runs another program and filters the
   output from that program.  It may also place restraints on the 
   resources that the program uses, such as time, memory, or rate of 
   output.
 */

static int pid = 0;
void handle_child( )
{
int prog_stat;
if (!pid) {
    waitpid( pid, &prog_stat, WNOHANG );
    pid = 0;
    }
}

/* D
   lcrun - Runs a program as a child

   Input Arguments:
.  userpgm - User's program
.  args    - user's arguments
D */
main(argc, argv)
int argc;
char **argv;
{
char buf[SUT_MAX_BUF];
int read_pipe[2], write_pipe[2];
int prog_stat, n;

/* Perform the handshake */
if (SUT_Handshake( 0, 1, 0 )) 
    return SUT_HANDSHAKE_FAILED;
    
/* Prepare to read from child */
if((pipe(read_pipe) == -1) || 
   (pipe(write_pipe) == -1))
    return SUT_PIPE_FAILED;

/* Establish a signal handler for SIGCHLD */
signal( SIGCHLD, handle_child );

pid = fork();
switch (pid) {
    case -1:
    fprintf( stderr, "Failed to fork!\n" );
    exit(1);

    case 0: /* child */
    close(read_pipe[0]);  /* close unused portion of pipe */
    close(write_pipe[1]); /* close unused portion of pipe */
    close(1);                /*         close stdout         */
    dup2(read_pipe[1],1);    /*  read_pipe[1] is now stdout  */
    close(read_pipe[1]);  /*  now only duplicate remains  */
    close(0);                /*         close  stdin         */
    dup2(write_pipe[0],0);   /*  write_pipe[0] is now stdin  */
    close(write_pipe[0]); /*  now only duplicate remains  */
    execvp( argv[1], argv+1 );
    exit(1);

    default: /* parent */
    close(read_pipe[1]);  /* close unused portion of pipe */
    close(write_pipe[0]); /* close unused portion of pipe */
    /* Just fall through and wait for input/output */
    ;
    }
/* Just input for now ... */
while (1) {
    n = read( read_pipe[0], buf, SUT_MAX_BUF );
    if (n < 0) {
	/* Error */
	perror( "lcrun: Error in reading from child" );
	break;
	}
    if (n == 0) {
	/* EOF */
	break;
	}
    /* This is where we'd look for \n and insert a given string ... */
    write( 1, buf, n );
    }
if (!pid) waitpid(pid, &prog_stat, WNOHANG );
return 0;
}

